import time
from serial import Serial
from adafruit_pm25.uart import PM25_UART

uart = Serial("/dev/ttyS0", baudrate=9600, timeout=0.25)
sensor = PM25_UART(uart, None)

# Convert a PM2.5 concentration value (in µg/m^3) to an AQI value
#
def pm25ToAqi(pm: int) -> int:
    assert pm >= 0, "PM concentration value is negative: " + str(pm) + "." + " It must be >=0."
    pm = round(float(pm), 1)
    if pm >= 325.5:
        return 500
    elif pm >= 225.5:
        return calculateAqi(pm, 500, 301, 325.4, 225.5)
    elif pm >= 125.5:
        return calculateAqi(pm, 300, 201, 225.4, 125.5)
    elif pm >= 55.5:
        return calculateAqi(pm, 200, 151, 125.4, 55.5)
    elif pm >= 35.5:
        return calculateAqi(pm, 150, 101, 55.4, 35.5)
    elif pm >= 9.1:
        return calculateAqi(pm, 100, 51, 35.4, 9.1)
    elif pm >= 0:
        return calculateAqi(pm, 50, 0, 9, 0)
    else:
        raise ValueError("Unexpected PM concentration value: " + str(pm))

# Convert a PM10 concentration value (in µg/m^3) to an AQI value
#
def pm100ToAqi(pm: int) -> int:
    assert pm >= 0, "PM concentration value is negative: " + str(pm) + "." + " It must be >=0."
    pm = int(pm)
    if pm >= 605:
        return 500
    elif pm >= 425:
        return calculateAqi(pm, 500, 301, 604, 425)
    elif pm >= 355:
        return calculateAqi(pm, 300, 201, 424, 355)
    elif pm >= 255:
        return calculateAqi(pm, 200, 151, 354, 255)
    elif pm >= 155:
        return calculateAqi(pm, 150, 101, 254, 155)
    elif pm >= 55:
        return calculateAqi(pm, 100, 51, 154, 55)
    elif pm >= 0:
        return calculateAqi(pm, 50, 0, 54, 0)
    else:
        raise ValueError("Unexpected PM concentration value: " + str(pm))

# https://en.wikipedia.org/wiki/Air_quality_index#Computing_the_AQI
# 
def calculateAqi(C: float, I_high: float, I_low: float, C_high: float, C_low: float) -> int:
    return round(((I_high - I_low)/(C_high - C_low)) * (C - C_low) + I_low)

# Convert an AQI value to an AQI category
#
def aqiToCategory(aqi: int) -> str:
    assert aqi >= 0, "AQI value is negative: " + str(aqi) + "." + " It must be >=0."
    aqiCategories = ("Hazardous", "Very Unhealthy", "Unhealthy",
                     "Unhealthy for Sensitive Groups", "Moderate", "Good")
    aqiBreakpoints = (301, 201, 151, 101, 51, 0)
    for index, aqiBreakpoint in enumerate(aqiBreakpoints):
        if aqi >= aqiBreakpoint:
            aqiCategoryIndex = index
    return aqiCategories[aqiCategoryIndex]

while True:
    try:
        time.sleep(2)
        pmData = sensor.read()
        pm10concentration      = pmData["pm10 env"]
        pm10to25concentration  = pmData["pm25 env"]
        pm25to100concentration = pmData["pm100 env"]
        pm25concentration  = pm10concentration + pm10to25concentration
        pm100concentration = pm25concentration + pm25to100concentration
        
        print("PM2.5 concentration:", pm25concentration, "μg/m^3")
        pm25Aqi = pm25ToAqi(pm25concentration)
        print("PM2.5 AQI:", pm25Aqi, aqiToCategory(pm25Aqi))
        
        print("PM10 concentration:" , pm100concentration, "μg/m^3")
        pm100aqi = pm100ToAqi(pm100concentration)
        print("PM10 AQI:", pm100aqi, aqiToCategory(pm100aqi))
        print("---------------------------------------")
    except RuntimeError:
        print("Unable to read from the sensor, retrying...")
        continue
    except KeyboardInterrupt:
        break




